//
//  ARsampleAppDelegate.m
//  ARsample
//
//  Created by hak on 5/14/10.
//  Copyright __MyCompanyName__ 2010. All rights reserved.
//

#import "ARsampleAppDelegate.h"
#import "EAGLView.h"

@implementation ARsampleAppDelegate

@synthesize window;
@synthesize glView;
@synthesize viewController;
@synthesize locationManager;
@synthesize accelerometerManager;

@synthesize _azimuth;
@synthesize _accerelometerX;
@synthesize _accerelometerY;
@synthesize _accerelometerZ;
@synthesize _magnetometerX;
@synthesize _magnetometerY;
@synthesize _magnetometerZ;


- (BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions
{
    //EAGL View
    [glView removeFromSuperview];
    [glView startAnimation];

    //Check camera availability
    if ([UIImagePickerController isSourceTypeAvailable:UIImagePickerControllerSourceTypeCamera]) {
        //Create imgpicker
        cameraController = [[UIImagePickerController alloc] init];
        cameraController.sourceType = UIImagePickerControllerSourceTypeCamera;
        
        //Hide buttons
        cameraController.showsCameraControls = NO;
        
        //Apply Transform
        float fMagnification = 480.f / (480.f - 96.f);
        cameraController.cameraViewTransform
            = CGAffineTransformScale(cameraController.cameraViewTransform, fMagnification, fMagnification);
        
        //Add overlay
        cameraController.cameraOverlayView = glView;
        
        //Show it
        [window addSubview:viewController.view];
        [viewController presentModalViewController:cameraController animated:NO];
    }
    
    //Location manager initialization
    locationManager = [[CLLocationManager alloc] init];
    
    locationManager.headingFilter = kCLHeadingFilterNone;
    locationManager.desiredAccuracy = kCLLocationAccuracyBest;
    locationManager.delegate = self;    
    
    [locationManager startUpdatingHeading];
    
    //Accereleometer
    accelerometerManager = [UIAccelerometer sharedAccelerometer];
    [accelerometerManager setUpdateInterval: 0.01 ];
    [accelerometerManager setDelegate: self ];
    
    return YES;
}

- (void)applicationWillResignActive:(UIApplication *)application
{
    [glView stopAnimation];
}

- (void)applicationDidBecomeActive:(UIApplication *)application
{
    [glView startAnimation];
}

- (void)applicationWillTerminate:(UIApplication *)application
{
    [glView stopAnimation];
}

- (void)dealloc
{
    [window release];
    [glView release];
    [cameraController release];
    [locationManager release];

    [super dealloc];
}

//-------------------------------------------------------------------------------------------
//Location management
//-------------------------------------------------------------------------------------------
- (void)locationManager:(CLLocationManager *)manager didUpdateHeading:(CLHeading *)newHeading
{
	_azimuth = fmod(newHeading.magneticHeading, 360.0) * (2 * (M_PI / 360.0));
    _magnetometerX = (newHeading.x * kFilteringFactor) + ( _magnetometerX * (1.0 - kFilteringFactor));
    _magnetometerY = (newHeading.y * kFilteringFactor) + ( _magnetometerY * (1.0 - kFilteringFactor));
    _magnetometerZ = (newHeading.z * kFilteringFactor) + ( _magnetometerZ * (1.0 - kFilteringFactor));
}

- (BOOL)locationManagerShouldDisplayHeadingCalibration:(CLLocationManager *)manager {	
	return YES;
}

- (void)locationManager:(CLLocationManager *)manager didUpdateToLocation:(CLLocation *)newLocation fromLocation:(CLLocation *)oldLocation {
}

- (void)locationManager:(CLLocationManager *)manager didFailWithError:(NSError *)error {
}

//-------------------------------------------------------------------------------------------
//Accelerometer management
//-------------------------------------------------------------------------------------------
- (void)accelerometer:(UIAccelerometer *)accelerometer didAccelerate:(UIAcceleration *)acceleration {
	
    //Low pass filter
	_accerelometerX = (acceleration.x * kFilteringFactor) + (_accerelometerX  * (1.0 - kFilteringFactor));
	_accerelometerY = (acceleration.y * kFilteringFactor) + (_accerelometerY  * (1.0 - kFilteringFactor));
	_accerelometerZ = (acceleration.z * kFilteringFactor) + (_accerelometerZ  * (1.0 - kFilteringFactor));

}

@end
