//
//  GLUtility.h
//  ARsample
//
//  Created by hakuroum on 5/16/10.
//  Copyright 2010 __MyCompanyName__. All rights reserved.
//

#import <Foundation/Foundation.h>

#import <OpenGLES/ES1/gl.h>
#import <OpenGLES/ES1/glext.h>

#ifdef __cplusplus
extern "C" {
#endif

void perspective(float fovy, float aspect, float zNear, float zFar);
void setCamera(GLfloat eyex, GLfloat eyey, GLfloat eyez,
			   GLfloat centerx, GLfloat centery, GLfloat centerz,
			   GLfloat upx, GLfloat upy, GLfloat upz);
void setCameraFromHeadPitch( GLfloat fHeading, GLfloat fPitch );
void setCameraYPR( GLfloat fYaw, GLfloat fPitch, GLfloat fRoll );
    
class CQuaternion {
private:
    GLfloat m_x;
    GLfloat m_y;
    GLfloat m_z;
    GLfloat m_w;
public:
    void createFromAxisAngle(GLfloat x, GLfloat y, GLfloat z, GLfloat degrees)
    {
        // First we want to convert the degrees to radians
        // since the angle is assumed to be in radians
        GLfloat angle = GLfloat((degrees / 180.0f) * M_PI);
        
        // Here we calculate the sin( theta / 2) once for optimization
        GLfloat result = (GLfloat)sinf( angle / 2.0f );
        
        // Calcualte the w value by cos( theta / 2 )
        m_w = (GLfloat)cosf( angle / 2.0f );
        
        // Calculate the x, y and z of the quaternion
        m_x = GLfloat(x * result);
        m_y = GLfloat(y * result);
        m_z = GLfloat(z * result);
    }
    
    void create(GLfloat x, GLfloat y, GLfloat z, GLfloat rad)
    {
        GLfloat angle = rad;
        
        // Here we calculate the sin( theta / 2) once for optimization
        GLfloat result = (GLfloat)sinf( angle * 0.5f );
        
        // Calcualte the w value by cos( theta / 2 )
        m_w = (GLfloat)cosf( angle * 0.5f );
        
        // Calculate the x, y and z of the quaternion
        m_x = GLfloat(x * result);
        m_y = GLfloat(y * result);
        m_z = GLfloat(z * result);
    }

    void createFromYawPitchRoll(GLfloat yaw, GLfloat pitch, GLfloat roll)
    {
        GLfloat num9 = roll * 0.5f;
        GLfloat num6 = sinf( num9 );
        GLfloat num5 = cosf( num9 );
        GLfloat num8 = pitch * 0.5f;
        GLfloat num4 = sinf( num8 );
        GLfloat num3 = cosf( num8 );
        GLfloat num7 = yaw * 0.5f;
        GLfloat num2 = sinf( num7 );
        GLfloat num = cosf( num7 );
        m_x = ((num * num4) * num5) + ((num2 * num3) * num6);
        m_y = ((num2 * num3) * num5) - ((num * num4) * num6);
        m_z = -((num * num3) * num6) + ((num2 * num4) * num5);
        m_w = ((num * num3) * num5) + ((num2 * num4) * num6);
    }
    
    void createMatrix(GLfloat *pMatrix)
    {
        // Make sure the matrix has allocated memory to store the rotation data
        if(!pMatrix) return;
        
        // First row
        pMatrix[ 0] = 1.0f - 2.0f * ( m_y * m_y + m_z * m_z );
        pMatrix[ 1] = 2.0f * (m_x * m_y + m_z * m_w);
        pMatrix[ 2] = 2.0f * (m_x * m_z - m_y * m_w);
        pMatrix[ 3] = 0.0f;
        
        // Second row
        pMatrix[ 4] = 2.0f * ( m_x * m_y - m_z * m_w );
        pMatrix[ 5] = 1.0f - 2.0f * ( m_x * m_x + m_z * m_z );
        pMatrix[ 6] = 2.0f * (m_z * m_y + m_x * m_w );
        pMatrix[ 7] = 0.0f;
        
        // Third row
        pMatrix[ 8] = 2.0f * ( m_x * m_z + m_y * m_w );
        pMatrix[ 9] = 2.0f * ( m_y * m_z - m_x * m_w );
        pMatrix[10] = 1.0f - 2.0f * ( m_x * m_x + m_y * m_y );
        pMatrix[11] = 0.0f;
        
        // Fourth row
        pMatrix[12] = 0;
        pMatrix[13] = 0;
        pMatrix[14] = 0;
        pMatrix[15] = 1.0f;
        
        // Now pMatrix[] is a 4x4 homogeneous matrix that can be applied to an OpenGL Matrix
    }

    CQuaternion operator *(CQuaternion q)
    {
        CQuaternion r;
        
        r.m_w = m_w*q.m_w - m_x*q.m_x - m_y*q.m_y - m_z*q.m_z;
        r.m_x = m_w*q.m_x + m_x*q.m_w + m_y*q.m_z - m_z*q.m_y;
        r.m_y = m_w*q.m_y + m_y*q.m_w + m_z*q.m_x - m_x*q.m_z;
        r.m_z = m_w*q.m_z + m_z*q.m_w + m_x*q.m_y - m_y*q.m_x;
        
        return(r);
    }
    
};


#ifdef __cplusplus
};
#endif
